MODULE halo_mng
   !!======================================================================
   !!                       ***  MODULE  halo_mng  ***
   !! Ocean numerics:  massively parallel processing library
   !!=====================================================================
   !! History :  OPA  !  1994  (M. Guyon, J. Escobar, M. Imbard)
   !Original code
   !!            4.0  !  2019  (CMCC - ASC)  initial version of halo management module
   !in_out_manager
   !!----------------------------------------------------------------------

   USE dom_oce       ! ocean space and time domain
   USE lbclnk        ! ocean lateral boundary condition (or mpp link) 

   IMPLICIT NONE
   PRIVATE

   INTERFACE halo_mng_resize
      MODULE PROCEDURE halo_mng_resize_2D, halo_mng_resize_3d, halo_mng_resize_4d
   END INTERFACE

   PUBLIC halo_mng_resize
   PUBLIC halo_mng_init
   PUBLIC halo_mng_set

   INTEGER :: jpi_1, jpj_1
   INTEGER :: jpimax_1, jpjmax_1
   INTEGER :: nlci_1, nlcj_1
   INTEGER :: jplbi_1, jplbj_1
CONTAINS

   SUBROUTINE halo_mng_init( )

        jpi_1 = jpi
        jpj_1 = jpj

        nlci_1 = nlci
        nlcj_1 = nlcj

        jplbi_1 = 1 
        jplbj_1 = 1 
        
        jplbi = 1 
        jplbj = 1 

   		jpimax_1 = jpimax
   		jpjmax_1 = jpjmax

   END SUBROUTINE halo_mng_init

   SUBROUTINE halo_mng_set( khls )
   
        INTEGER, INTENT(in   )    ::   khls

        nn_hls = khls
        jpi = jpi_1 + khls -1
        jpj = jpj_1 + khls -1

        nlci = nlci_1 + khls -1
        nlcj = nlcj_1 + khls -1
        
        jplbi = jplbi_1 - khls +1 
        jplbj = jplbj_1 - khls +1 
        
        jpimax = jpimax_1 + khls -1
        jpjmax = jpjmax_1 + khls -1

   END SUBROUTINE halo_mng_set
   
	SUBROUTINE halo_mng_resize_2D(pta, cdna, psgn, fillval)
	
		REAL(wp), POINTER, DIMENSION(:,:) :: pta
		CHARACTER(len=1), INTENT(in)  :: cdna
		REAL(wp), INTENT(in)  :: psgn
		REAL(wp), OPTIONAL, INTENT(in ) :: fillval
		REAL(wp), POINTER, DIMENSION(:,:) :: zpta
		INTEGER :: offset
		INTEGER :: pta_size_i, pta_size_j, exp_size_i, exp_size_j

		pta_size_i = SIZE(pta,1)
		pta_size_j = SIZE(pta,2)
		exp_size_i = jpi - jplbi + 1
		exp_size_j = jpj - jplbj + 1
		
		! check if the current size of pta is equal to the current expected dimension
		IF (pta_size_i .ne. exp_size_i) THEN
			ALLOCATE (zpta(jplbi:jpi, jplbj:jpj))
			offset = (exp_size_i - pta_size_i) / 2 

			IF (pta_size_i .lt. exp_size_i) THEN
				zpta (offset+jplbi : offset+pta_size_i+jplbi-1, offset+jplbj : offset+pta_size_j+jplbj-1) = pta
			ELSE
				zpta = pta(jplbi : jpi, jplbj : jpj)
			END IF
			CALL lbc_lnk( 'halo_mng_resize_2D', zpta, cdna, psgn, pfillval=fillval)
			DEALLOCATE(pta)
			pta => zpta
		END IF
		
	END SUBROUTINE halo_mng_resize_2D

	SUBROUTINE halo_mng_resize_3D(pta, cdna, psgn, fillval)
	
		REAL(wp), POINTER, DIMENSION(:,:,:) :: pta
		CHARACTER(len=1), INTENT(in)  :: cdna
		REAL(wp), INTENT(in)  :: psgn
		REAL(wp), OPTIONAL, INTENT(in ) :: fillval
		REAL(wp), POINTER, DIMENSION(:,:,:) :: zpta
		INTEGER :: offset
		INTEGER :: pta_size_i, pta_size_j, exp_size_i, exp_size_j

		pta_size_i = SIZE(pta,1)
		pta_size_j = SIZE(pta,2)
		exp_size_i = jpi - jplbi + 1
		exp_size_j = jpj - jplbj + 1
		
		! check if the current size of pta is equal to the current expected dimension
		IF (pta_size_i .ne. exp_size_i) THEN
			ALLOCATE (zpta(jplbi:jpi, jplbj:jpj, jpk))
			offset = (exp_size_i - pta_size_i) / 2 

			IF (pta_size_i .lt. exp_size_i) THEN
				zpta (offset+jplbi : offset+pta_size_i+jplbi-1, offset+jplbj : offset+pta_size_j+jplbj-1, :) = pta
			ELSE
				zpta = pta(jplbi : jpi, jplbj : jpj, :)
			END IF
			CALL lbc_lnk( 'halo_mng_resize_3D', zpta, cdna, psgn, pfillval=fillval)
			DEALLOCATE(pta)
			pta => zpta
		END IF
		
	END SUBROUTINE halo_mng_resize_3D

	SUBROUTINE halo_mng_resize_4D(pta, cdna, psgn, fillval, kjpt)
	
		REAL(wp), POINTER, DIMENSION(:,:,:,:) :: pta
		CHARACTER(len=1), INTENT(in)  :: cdna
		REAL(wp), INTENT(in)  :: psgn
		REAL(wp), OPTIONAL, INTENT(in) :: fillval
		INTEGER , OPTIONAL, INTENT(in) ::   kjpt            ! number of tracers
		REAL(wp), POINTER, DIMENSION(:,:,:,:) :: zpta
		INTEGER :: offset
		INTEGER :: pta_size_i, pta_size_j, exp_size_i, exp_size_j

		pta_size_i = SIZE(pta,1)
		pta_size_j = SIZE(pta,2)
		exp_size_i = jpi - jplbi + 1
		exp_size_j = jpj - jplbj + 1
		
		! check if the current size of pta is equal to the current expected dimension
		IF (pta_size_i .ne. exp_size_i) THEN
			ALLOCATE (zpta(jplbi:jpi, jplbj:jpj, jpk, kjpt))
			offset = (exp_size_i - pta_size_i) / 2 

			IF (pta_size_i .lt. exp_size_i) THEN
				zpta (offset+jplbi : offset+pta_size_i+jplbi-1, offset+jplbj : offset+pta_size_j+jplbj-1, :, :) = pta
			ELSE
				zpta = pta(jplbi : jpi, jplbj : jpj, :, :)
			END IF
			CALL lbc_lnk( 'halo_mng_resize_4D', zpta, cdna, psgn, pfillval=fillval)
			DEALLOCATE(pta)
			pta => zpta
		END IF
		
	END SUBROUTINE halo_mng_resize_4D
	
END MODULE
